@@ -1,20 +1,20 @@
-# amazon.rb $Revision: 1.27.2.5 $: Making link with image to Amazon using Amazon ECS.
+# amazon.rb $Revision: 1.47 $: Making link with image to Amazon using Amazon ECS.
 #
 # see document: #{@lang}/amazon.rb
 #
 # Copyright (C) 2005 TADA Tadashi <sho@spc.gr.jp>
 # You can redistribute it and/or modify it under GPL2.
 #
-require 'net/http'
+require 'open-uri'
 require 'timeout'
+require 'rexml/document'
+require 'nkf'
 
 # do not change these variables
 @amazon_subscription_id = '1CVA98NEF1G753PFESR2'
 @amazon_require_version = '2005-07-26'
 
 def amazon_call_ecs( asin )
-	limittime = 10
-
 	aid =  @conf['amazon.aid'] || ''
 	aid = 'cshs-22' if aid.length == 0
 
@@ -27,68 +27,49 @@ def amazon_call_ecs( asin )
 	url << "&ResponseGroup=Medium"
 	url << "&Version=#{@amazon_require_version}"
 
-	proxy_host = nil
-	proxy_port = 8080
-	if /^([^:]+):(\d+)$/ =~ @conf['amazon.proxy'] then
-		proxy_host = $1
-		proxy_port = $2.to_i
+	proxy = nil
+	if @conf['amazon.proxy'] and @conf['amazon.proxy'].length > 0 then
+		proxy = @conf['amazon.proxy']
+		proxy = 'http://' + proxy unless proxy =~ /^https?:/
 	end
 
-	xml = nil
 	timeout( 10 ) do
-		begin
-			if %r|http://([^:/]*):?(\d*)(/.*)| =~ url then
-				host = $1
-				port = $2.to_i
-				path = $3
-				port = 80 if port == 0
-			end
-			Net::HTTP.version_1_1
-			Net::HTTP.Proxy( proxy_host.untaint, proxy_port.untaint ).start( host.untaint, port.untaint ) do |http|
-				response, = http.get( path )
-				xml = response.body
-			end
-		rescue Net::ProtoRetriableError => err
-			item_url = err.response['location']
-			retry
-		rescue
-			raise 'getting item was failed'
-		end
+		open( url, :proxy => proxy ) {|f| f.read}
 	end
-	xml
 end
 
 def amazon_to_html( item, with_image = true, label = nil, pos = 'amazon' )
 	with_image = false if @mode == 'categoryview'
-	author = ''
-	item.scan( %r|<Author>(.*?)</Author>|m ) do |a|
-		author << a[0] << '/'
+	begin
+		author = ''
+		item.elements.each( '*/Author' ) do |a|
+			author << a.text << '/'
+		end
+		author = "(#{NKF::nkf '-We', author.chop!})"
+	rescue
+		author = ''
 	end
-	author = author.empty? ? '' : "(#{author.chop})"
 
 	unless label then
-		title = item.scan( %r|<Title>(.*?)</Title>|m )[0][0]
-		label = "#{title}#{author}"
+		label = %Q|#{NKF::nkf '-We', item.elements.to_a( '*/Title' )[0].text}#{author}|
 	end
 
 	image = ''
 	if with_image then
-		size = case @conf['amazon.imgsize']
+		begin
+			size = case @conf['amazon.imgsize']
 			when 0; 'Large' 
 			when 2; 'Small'
 			else;   'Medium'
-		end
-		begin
-			image_item = item.scan( %r|<#{size}Image>(.*?)</#{size}Image>|m )[0][0]
-			url = image_item.scan( %r|<URL>(.*?)</URL>|m )[0][0]
-			w = image_item.scan( %r|<Width.*?>(.*?)</Width>|m )[0][0]
-			h = image_item.scan( %r|<Height.*?>(.*?)</Height>|m )[0][0]
+			end
 			image = <<-HTML
 			<img class="#{pos}"
-			src="#{url}" height="#{h}" width="#{w}"
-			alt="#{label}" title="#{label}">
+			src="#{item.elements.to_a( "#{size}Image/URL" )[0].text}"
+			height="#{item.elements.to_a( "#{size}Image/Height" )[0].text}"
+			width="#{item.elements.to_a( "#{size}Image/Width" )[0].text}"
+			alt="#{CGI::escapeHTML(label)}" title="#{CGI::escapeHTML(label)}">
 			HTML
-		rescue NameError
+		rescue
 			if @conf['amazon.nodefault'] then
 				image = CGI::escapeHTML(label)
 			else
@@ -119,8 +100,7 @@ def amazon_to_html( item, with_image = true, label = nil, pos = 'amazon' )
 		label = ''
 	end
 
-	detail = item.scan( %r|<DetailPageURL>(.*?)</DetailPageURL>|m )[0][0]
-	%Q|<a href="#{detail}">#{image}#{label}</a>|
+	%Q|<a href="#{item.elements.to_a( 'DetailPageURL' )[0].text}">#{image}#{CGI::escapeHTML(label)}</a>|
 end
 
 def amazon_secure_html( asin, with_image, label, pos = 'amazon' )
@@ -157,19 +137,23 @@ def amazon_get( asin, with_image = true, label = nil, pos = 'amazon' )
 			cache = "#{@cache_path}/amazon"
 			Dir::mkdir( cache ) unless File::directory?( cache )
 			begin
-				xml = File::open( "#{cache}/#{asin}.xml", 'r' ) {|f| f.read}
+				xml = File::read( "#{cache}/#{asin}.xml" )
 			rescue Errno::ENOENT
 				xml =  amazon_call_ecs( asin )
 				File::open( "#{cache}/#{asin}.xml", 'wb' ) {|f| f.write( xml )}
 			end
-			xml = @amazon_encoder.call( xml )
-
-			item = xml.scan( %r|<Item>(.*)</Item>|m )[0][0]
+			doc = REXML::Document::new( xml ).root
+			item = doc.elements.to_a( '*/Item' )[0]
 			amazon_to_html( item, with_image, label, pos )
-		rescue TimeoutError
+		rescue Timeout::Error
 			asin
-		rescue
-			"#{label ? label : asin}"
+		rescue NoMethodError
+			if item == nil then
+				message = doc.elements.to_a( 'Items/Request/Errors/Error/Message' )[0].text
+				"#{label ? label : asin}<!--#{NKF::nkf( '-We', message )}-->"
+			else
+				"#{label ? label : asin}<!--#{$!}\n#{$@.join ' / '}-->"
+			end
 		end
 	end
 end
