@@ -1,5 +1,5 @@
 =begin
-= A little bit more powerful display of referrers((-$Id: disp_referrer.rb,v 1.9.2.1 2005/07/20 08:25:02 tadatadashi Exp $-))
+= A little bit more powerful display of referrers((-$Id: disp_referrer.rb,v 1.16 2005/10/15 12:27:54 zunda Exp $-))
 English resource
 
 == Copyright notice
@@ -69,26 +69,22 @@ Disp_referrer2_with_Nora = <<'_END'.taint
 </p>
 _END
 Disp_referrer2_without_Nora = <<'_END'.taint
-<p>
+<!-- p>
 	Please install the 
-	<a href="http://raa.ruby-lang.org/list.rhtml?name=Nora">Nora</a>
-	library if you feel the pages show too slowly.
-</p>
-_END
-Disp_referrer2_scanned_urls = <<-'_END'.taint
-<p>%d URL(s) in the diary are added to the cache.</p>
-_END
-Disp_referrer2_updated_urls = <<-'_END'.taint
-<p>%d URL(s) in the cache are updated.</p>
+	<a href="http://www.moonwolf.com/ruby/archive/nora-20040830.tar.gz">Nora
+	library</a> if you feel the pages show too slowly.
+</p -->
 _END
 Disp_referrer2_cache_info = <<'_END'.taint
 <p>
-	The cache has %2$s URL(s) in %1$s byte(s).
+	Total size of the caches is %1$s byte(s).
 </p>
 _END
 Disp_referrer2_update_info = <<'_END'.taint
 <p>
-	Please update the cache
+	Please clear the cache by checking this box:
+  <input name="dr2.cache.update" value="force" type="checkbox">clear the cache,
+	and clicking OK
 	after editing the <a href="%1$s">today's link</a> lists.
 </p>
 _END
@@ -175,20 +171,16 @@ class DispRef2SetupIF
 				<td><input name="dr2.no_cache" value="false" type="radio"#{' checked'if not @setup['no_cache']}>Use
 				<td><input name="dr2.no_cache" value="true" type="radio"#{' checked'if @setup['no_cache']}>Don't use
 				cache.
-			</table>
-			<table>
 			<tr>
-				<td><input name="dr2.cache.update" value="force" type="radio">update
-				<td><input name="dr2.cache.update" value="auto" type="radio" checked>update if needed
-				<td><input name="dr2.cache.update" value="never" type="radio">don't update
+				<td>Limit cache size to
+				<td colspan="3"><input name="dr2.cache_max_size" value="#{@setup['cache_max_size']}" type="text">Bytes.
+			<tr>
+				<td><input name="dr2.cache.update" value="force" type="radio">clear
+				<td><input name="dr2.cache.update" value="auto" type="radio" checked>clear if needed
+				<td><input name="dr2.cache.update" value="never" type="radio">don't clear
 				the cache at this time.
 			</table>
-			<p>
-				Updating the cache sometimes takes time.
-				Please wait after clicking the OK button.
-				On the other hand, referrers might not shown correctly when
-				the cache is not updated.
-			</p>
+			<p>Cache size limit is an approximation. There is a chance that the cache size is bigger than the configured value. Setting the limit to zero disalbes limitation. K or M can be suffixed meaning Kbytes or Mbytes.</p>
 		_HTML
 		end # unless @setup.secure
 		r
@@ -197,25 +189,18 @@ class DispRef2SetupIF
 	# shows URL list to be added to the referer_table or no_referer
 	def show_unknown_list
 		if @setup.secure then
-			urls = DispRef2Latest_cache.unknown_urls
-		elsif @setup['no_cache'] then
 			urls = DispRef2Latest.new( @cgi, 'latest.rhtml', @conf, @setup ).unknown_urls
 		else
-			urls = DispRef2Cache.new( @setup ).unknown_urls
+			urls = DispRef2Cache.new( @setup ).urls( DispRef2URL::Unknown ).keys
+			if urls.size == 0 then
+				urls = DispRef2Latest.new( @cgi, 'latest.rhtml', @conf, @setup ).unknown_urls
+			end
 		end
+		urls.reject!{ |url| DispRef2String::url_match?( url, @setup['reflist.ignore_urls'] ) }
 		r = <<-_HTML
 			<h3>URL Conversion</h3>
 			<input name="dr2.current_mode" value="#{RefList}" type="hidden">
-		_HTML
-		if @cache then
-			r << "<p>Picking up #{@setup['unknown.label']} from the cache."
-		else
-			r << "<p>Picking up #{@setup['unknown.label']} from the latest view."
-		end
-		r << <<-_HTML
-			URLs that match the Excluding list or the Ignore list are not
-			listed here.
-		</p>
+			<p>URLs that match the Ignore list are not listed here.</p>
 		<p>
 			If you don't want to see the URLs that you neither put into the
 			Conversion list or the Excluding list can be put in the Ignore
@@ -242,7 +227,7 @@ class DispRef2SetupIF
 					a script fragment like &quot;sprintf('[tdiary:%d]', $1.to_i+1)&quot;.
 				</p>
 			_HTML
-			if ENV['AUTH_TYPE'] and ENV['REMOTE_USER'] and @setup['configure.use_link'] then
+			if @cgi.auth_type and @cgi.remote_user and @setup['configure.use_link'] then
 				r << <<-_HTML
 					<p>
 						[NOTE] Be aware that by clicking the URLs below, the author
@@ -261,7 +246,7 @@ class DispRef2SetupIF
 			i = 0
 			urls.sort.each do |url|
 				shown_url = DispRef2String::escapeHTML( @setup.to_native( DispRef2String::unescape( url ) ) )
-				if ENV['AUTH_TYPE'] and ENV['REMOTE_USER'] and @setup['configure.use_link'] then
+				if @cgi.auth_type and @cgi.remote_user and @setup['configure.use_link'] then
 					r << "<dt><a href=\"#{url}\">#{shown_url}</a>"
 				else
 					r << "<dt>#{shown_url}"
@@ -280,14 +265,6 @@ class DispRef2SetupIF
 				<input name="dr2.urls" type="hidden" value="#{i}">
 				</dl>
 			_HTML
-			unless @setup.secure or @setup['no_cache'] then
-				r << <<-_HTML
-					<p>
-						Updating the cache might take some time. Please wait after
-						clicking the OK button.
-					</p>
-				_HTML
-			end
 		else
 			r << <<-_HTML
 				<p>Currently there is no #{@setup['unknown.label']}.</p>
