@@ -1,4 +1,4 @@
-# image.rb $Revision: 1.23.2.4 $
+# image.rb $Revision: 1.31 $
 # -pv-
 # 
 # ̾:
@@ -39,6 +39,10 @@
 #  @options['image.maxsize']
 #     1礢κХȿ̵10000
 #     @secure = trueΤͭ
+#  @options['image.maxwidth']
+#     sizeꤷʤä˻Ǥκɽ̵nil
+#     ɽΤӤ˥ե륢ΤǡŤʤ뤫?
+#     @secure = true̵
 #
 # 饤󥹤ˤĤ:
 # Copyright (c) 2002,2003 Daisuke Kato <dai@kato-agri.com>
@@ -47,24 +51,6 @@
 # Distributed under the GPL
 #
 
-=begin Changelog
-See recent ChangeLog to plugins collections.
-
-2003-04-23 Yoshimi KURUMA <yoshimik@iris.dti.ne.jp>
-	* image.rb: add JavaScript for insert plugin tag into diary.
-
-2003-04-23 Daisuke Kato <dai@kato-agri.com>
-	* tuning around form tag.
-
-2003-04-23 Yoshimi KURUMA <yoshimik@iris.dti.ne.jp>
-	* Now img tag includes class="photo".
-	* New Option. image.maxnum, image.maxsize.
-	* fine tuning around form tag.
-
-2003-04-22 Yoshimi KURUMA <yoshimik@iris.dti.ne.jp>
-	* version 0.5 first form_proc version.
-=end
-
 unless @resource_loaded then
 	def image_error_num( max ); "1#{max}ޤǤǤפʲƤɲäƤ"; end
 	def image_error_size( max ); "κ祵#{max}ХȤޤǤǤ"; end
@@ -92,8 +78,15 @@ def image( id, alt = 'image', thumbnail = nil, size = nil, place = 'photo' )
 		else
 			size = " width=\"#{size.to_i}\""
 		end
-	else
-		size = ""
+	elsif @image_maxwidth and not @conf.secure then
+		File::open( "#{@image_dir}/#{image}".untaint ) do |f|
+			t, w, h = image_info( f )
+			if w > @image_maxwidth then
+				size = %Q[ width="#{@image_maxwidth}"]
+			else
+				size = ""
+			end
+		end
 	end
 	if thumbnail then
 	   	%Q[<a href="#{@image_url}/#{image}"><img class="#{place}" src="#{@image_url}/#{image_t}" alt="#{alt}" title="#{alt}"#{size}></a>]
@@ -126,6 +119,7 @@ end
 @image_dir.chop! if /\/$/ =~ @image_dir
 @image_url = @options && @options['image.url'] || "#{@conf.base_url}images/"
 @image_url.chop! if /\/$/ =~ @image_url
+@image_maxwidth = @options && @options['image.maxwidth'] || nil
 
 add_body_enter_proc do |date|	
    @image_date = date.strftime( "%Y%m%d" )
@@ -144,11 +138,11 @@ def image_info( f )
 	sig = f.read( 24 )
 	if /\A\x89PNG\x0D\x0A\x1A\x0A(....)IHDR(........)/on =~ sig
 		image_type = 'png'
-		image_height, image_width = $2.unpack( 'NN' )
+		image_width, image_height = $2.unpack( 'NN' )
 
 	elsif /\AGIF8[79]a(....)/on =~ sig
 		image_type   = 'gif'
-		image_height, image_width = $1.unpack( 'vv' )
+		image_width, image_height = $1.unpack( 'vv' )
 
 	elsif /\A\xFF\xD8/on =~ sig
 		image_type = 'jpg'
@@ -160,7 +154,7 @@ def image_info( f )
 			data_size = data[2,2].unpack( 'n' ).first + 2
 			case data[1]
 			when 0xc0, 0xc1, 0xc2, 0xc3, 0xc5, 0xc6, 0xc7, 0xc9, 0xca, 0xcb, 0xcd, 0xce, 0xcf
-				image_width, image_height = data[5,4].unpack('nn')
+				image_height, image_width = data[5,4].unpack('nn')
 				break
 			else
 				if data.size < data_size
@@ -174,7 +168,7 @@ def image_info( f )
 		end
 	end
 
-	return image_type, image_height, image_width
+	return image_type, image_width, image_height
 end
 
 def image_ext
