@@ -1,7 +1,6 @@
 #!/usr/bin/env ruby
-$KCODE = 'n'
 
-# squeeze.rb $Revision: 1.18.2.2 $
+# squeeze.rb $Revision: 1.23 $
 #
 # Create daily HTML file from tDiary database.
 #
@@ -16,8 +15,6 @@ $KCODE = 'n'
 # version 1.0.4 by TADA Tadashi <sho@spc.gr.jp> with GPL2.
 #
 
-$KCODE= 'e'
-
 mode = defined?(TDiary) ? "PLUGIN" : ENV["REQUEST_METHOD"]? "CGI" : "CMD"
 
 if mode == "CMD" || mode == "CGI"
@@ -31,7 +28,7 @@ if mode == "CMD" || mode == "CGI"
 
 	if mode == "CMD"
 		def usage
-			puts "squeeze $Revision: 1.18.2.2 $"
+			puts "squeeze $Revision: 1.23 $"
 			puts " making html files from tDiary's database."
 			puts " usage: ruby squeeze.rb [-p <tDiary path>] [-c <tdiary.conf path>] [-a] [-s] [-x suffix] <dest path>"
 			exit
@@ -71,6 +68,7 @@ if mode == "CMD" || mode == "CGI"
 		tdiary_conf = tdiary_path unless tdiary_conf
 		Dir::chdir( tdiary_conf )
 		ARGV << '' # dummy argument against cgi.rb offline mode.
+		$:.unshift tdiary_path
 	else
 		@options = Hash.new
 		File::readlines("tdiary.conf").each {|item| 
@@ -87,14 +85,14 @@ if mode == "CMD" || mode == "CGI"
 		compat = @options['squeeze.compat_path'] || @options['yasqueeze.compat_path']
 
 		if FileTest::symlink?( __FILE__ ) then
-			tdiary_path = File::dirname( File::readlink( __FILE__ ) )
+			org_path = File::dirname( File::readlink( __FILE__ ) )
 		else
-			tdiary_path = File::dirname( __FILE__ )
+			org_path = File::dirname( __FILE__ )
 		end
+		$:.unshift( org_path.untaint )
 	end
 
 	begin
-		$:.unshift tdiary_path
 		require "tdiary"
 	rescue LoadError
 		$stderr.print "squeeze.rb: cannot load tdiary.rb. <#{tdiary_path}/tdiary>\n"
@@ -110,7 +108,9 @@ module TDiary
 		def initialize(diary, dest, all_data, compat, conf, suffix)
 			@ignore_parser_cache = true
 	
-			super(CGI::new, 'day.rhtml', conf)
+			cgi = CGI::new
+			def cgi.referer; nil; end
+			super( cgi, 'day.rhtml', conf )
 			@diary = diary
 			@date = diary.date
 			@diaries = {@date.strftime('%Y%m%d') => @diary} if @diaries.size == 0
@@ -165,7 +165,9 @@ module TDiary
 		def initialize(dest, all_data, compat, conf, suffix)
 			@ignore_parser_cache = true
 	
-			super(CGI::new, 'day.rhtml', conf)
+			cgi = CGI::new
+			def cgi.referer; nil; end
+			super( cgi, 'day.rhtml', conf )
 			calendar
 			@years.keys.sort.each do |year|
 				print "(#{year.to_s}/) "
@@ -192,14 +194,16 @@ if mode == "CGI" || mode == "CMD"
 			</head>
 			<body><div style="text-align:center">
 			<h1>Squeeze for tDiary</h1>
-			<p>$Revision: 1.18.2.2 $</p>
+			<p>$Revision: 1.23 $</p>
 			<p>Copyright (C) 2002 MUTOH Masao&lt;mutoh@highway.ne.jp&gt;</p></div>
 			<br><br>Start!</p><hr>
 		]
 	end
 
 	begin
-		conf = TDiary::Config::new
+		require 'cgi'
+		cgi = CGI.new
+		conf = TDiary::Config::new(cgi)
 		conf.header = ''
 		conf.footer = ''
 		conf.show_comment = true
