@@ -1,6 +1,9 @@
 #
 # 00default.rb: default plugins 
-# $Revision: 1.71.2.1 $
+# $Revision: 1.97 $
+#
+# Copyright (C) 2001-2005, TADA Tadashi <sho@spc.gr.jp>
+# You can redistribute it and/or modify it under GPL2.
 #
 
 #
@@ -13,36 +16,103 @@ def navi
 	result << %Q[</div>]
 end
 
+def navi_item( link, label )
+	%Q[<span class="adminmenu"><a href="#{link}">#{label}</a></span>\n]
+end
+
 def navi_user
-	result = ''
-	result << %Q[<span class="adminmenu"><a href="#{@index_page}">#{navi_index}</a></span>\n] unless @index_page.empty?
-	if @mode == 'day' then
-		if @prev_day then
-			result << %Q[<span class="adminmenu"><a href="#{@index}#{anchor @prev_day}">&laquo;#{navi_prev_diary Time::local(*@prev_day.scan(/^(\d{4})(\d\d)(\d\d)$/)[0])}</a></span>\n]
-		end
+	result = navi_user_default
 
-		result << %Q[<span class="adminmenu"><a href="#{@index}">#{navi_latest}</a></span>\n] unless @mode == 'latest'
-
-		if @next_day
-			result << %Q[<span class="adminmenu"><a href="#{@index}#{anchor @next_day}">#{navi_next_diary Time::local(*@next_day.scan(/^(\d{4})(\d\d)(\d\d)$/)[0])}&raquo;</a></span>\n]
-		end
-	elsif @mode == 'nyear'
-		result << %Q[<span class="adminmenu"><a href="#{@index}#{anchor @prev_day[4,4]}">&laquo;#{navi_prev_nyear Time::local(*@prev_day.scan(/^(\d{4})(\d\d)(\d\d)$/)[0])}</a></span>\n] if @prev_day
-		result << %Q[<span class="adminmenu"><a href="#{@index}">#{navi_latest}</a></span>\n] unless @mode == 'latest'
-		result << %Q[<span class="adminmenu"><a href="#{@index}#{anchor @next_day[4,4]}">#{navi_next_nyear Time::local(*@next_day.scan(/^(\d{4})(\d\d)(\d\d)$/)[0])}&raquo;</a></span>\n] if @next_day
+	case @mode
+	when 'latest'
+		result << navi_user_latest
+	when 'day'
+		result << navi_user_day
+	when 'month'
+		result << navi_user_month
+	when 'nyear'
+		result << navi_user_nyear
+	when 'edit'
+		result << navi_user_edit
 	else
-		result << %Q[<span class="adminmenu"><a href="#{@index}">#{navi_latest}</a></span>\n] unless @mode == 'latest'
+		result << navi_user_else
 	end
 	result
 end
 
+def navi_user_default
+	result = ''
+	result << navi_item( @index_page, navi_index ) unless @index_page.empty?
+	result
+end
+
+def navi_user_latest
+	result = ''
+	result << navi_item( "#{@index}#{anchor( @conf['ndays.prev'] + '-' + @conf.latest_limit.to_s )}", "&laquo;#{navi_prev_ndays}" ) if @conf['ndays.prev'] and not bot?
+	result << navi_item( @index, navi_latest ) if @cgi.params['date'][0]
+	result << navi_item( "#{@index}#{anchor( @conf['ndays.next'] + '-' + @conf.latest_limit.to_s )}", "#{navi_next_ndays}&raquo;" ) if @conf['ndays.next'] and not bot?
+	result
+end
+
+def navi_user_day
+	result = ''
+	result << navi_item( "#{@index}#{anchor @prev_day}", "&laquo;#{navi_prev_diary Time::local(*@prev_day.scan(/^(\d{4})(\d\d)(\d\d)$/)[0])}" ) if @prev_day
+	result << navi_item( @index, navi_latest )
+	result << navi_item( "#{@index}#{anchor @next_day}", "#{navi_next_diary Time::local(*@next_day.scan(/^(\d{4})(\d\d)(\d\d)$/)[0])}&raquo;" ) if @next_day
+	result
+end
+
+def navi_user_month
+	ym = []
+	@years.keys.each do |y|
+		ym += @years[y].collect {|m| y + m}
+	end
+	ym.sort!
+	now = @date.strftime( '%Y%m' )
+	return '' unless ym.index( now )
+	prev_month = ym.index( now ) == 0 ? nil : ym[ym.index( now )-1]
+	next_month = ym[ym.index( now )+1]
+
+	result = ''
+	result << navi_item( "#{@index}#{anchor( prev_month )}", "&laquo;#{navi_prev_month}" ) if prev_month and not bot?
+	result << navi_item( @index, navi_latest )
+	result << navi_item( "#{@index}#{anchor( next_month )}", "#{navi_next_month}&raquo;" ) if next_month and not bot?
+	result
+end
+
+def navi_user_nyear
+	result = ''
+	result << navi_item( "#{@index}#{anchor @prev_day[4,4]}", "&laquo;#{navi_prev_nyear Time::local(*@prev_day.scan(/^(\d{4})(\d\d)(\d\d)$/)[0])}" ) if @prev_day
+	result << navi_item( @index, navi_latest ) unless @mode == 'latest'
+	result << navi_item( "#{@index}#{anchor @next_day[4,4]}", "#{navi_next_nyear Time::local(*@next_day.scan(/^(\d{4})(\d\d)(\d\d)$/)[0])}&raquo;" ) if @next_day
+	result
+end
+
+def navi_user_edit
+	result = ''
+	if @prev_day then
+		a = @prev_day.scan( /^(\d{4})(\d\d)(\d\d)$/ ).flatten
+		result << navi_item( "#{@update}?edit=true;year=#{a[0]};month=#{a[1]};day=#{a[2]}", "&laquo;#{navi_prev_diary Time::local(*@prev_day.scan(/^(\d{4})(\d\d)(\d\d)$/)[0])}" )
+	end
+	result << navi_item( @index, navi_latest )
+	if @next_day then
+		a = @next_day.scan( /^(\d{4})(\d\d)(\d\d)$/ ).flatten
+		result << navi_item( "#{@update}?edit=true;year=#{a[0]};month=#{a[1]};day=#{a[2]}", "#{navi_next_diary Time::local(*@next_day.scan(/^(\d{4})(\d\d)(\d\d)$/)[0])}&raquo;" )
+	end
+	result
+end
+
+def navi_user_else
+	navi_item( @index, navi_latest )
+end
+
 def navi_admin
 	if @mode == 'day' then
-		result = %Q[<span class="adminmenu"><a href="#{@update}?edit=true;year=#{@date.year};month=#{@date.month};day=#{@date.day}">#{navi_edit}</a></span>\n]
+		result = navi_item( "#{@update}?edit=true;year=#{@date.year};month=#{@date.month};day=#{@date.day}", navi_edit )
 	else
-		result = %Q[<span class="adminmenu"><a href="#{@update}">#{navi_update}</a></span>\n]
+		result = navi_item( @update, navi_update )
 	end
-	result << %Q[<span class="adminmenu"><a href="#{@update}?conf=default">#{navi_preference}</a></span>\n] if /^(latest|month|day|comment|conf|nyear|category.*)$/ !~ @mode
+	result << navi_item( "#{@update}?conf=default", navi_preference ) if /^(latest|month|day|comment|conf|nyear|category.*)$/ !~ @mode
 	result
 end
 
@@ -113,6 +183,9 @@ add_header_proc do
 	#{author_name_tag}
 	#{author_mail_tag}
 	#{index_page_tag}
+	#{mobile_link_discovery}
+	#{icon_tag}
+	#{description_tag}
 	#{css_tag.chomp}
 	#{title_tag.chomp}
 	#{robot_control.chomp}
@@ -120,7 +193,7 @@ add_header_proc do
 end
 
 def calc_links
-	if @mode == 'day' or (@conf.mobile_agent? and /(latest|month|nyear)/ === @mode) then
+	if /day|edit/ =~ @mode or (@conf.mobile_agent? and /latest|month|nyear/ =~ @mode) then
 		years = []
 		@years.each do |k, v|
 			v.each do |m|
@@ -147,7 +220,7 @@ def calc_links
 		days.index( today ).times do |i|
 			@prev_day = days[days.index( today ) - i - 1]
 			break unless @prev_day
-			break if @diaries[@prev_day].visible?
+			break if (@mode == 'edit') or @diaries[@prev_day].visible?
 		end
 		if not @prev_day and prev_month then
 			y, m = prev_month.scan(/(\d{4})(\d\d)/)[0]
@@ -162,7 +235,7 @@ def calc_links
 		days.index( today ).times do |i|
 			@next_day = days[days.index( today ) + i + 1]
 			break unless @next_day
-			break if @diaries[@next_day].visible?
+			break if (@mode == 'edit') or @diaries[@next_day].visible?
 		end
 		if not @next_day and next_month then
 			y, m = next_month.scan(/(\d{4})(\d\d)/)[0]
@@ -237,12 +310,44 @@ def index_page_tag
 	result.chop.chop
 end
 
+def mobile_link_discovery
+	return '' unless /^(latest|day)$/ =~ @mode
+	uri = @conf.index.dup
+	uri[0, 0] = @conf.base_url if %r|^https?://|i !~ @conf.index
+	uri.gsub!( %r|/\./|, '/' )
+	if @mode == 'day' then
+		uri += anchor( @date.strftime( '%Y%m%d' ) )
+	end
+	%Q[<link rel="alternate" media="handheld" type="text/html" href="#{uri}">]
+end
+
+def icon_tag
+	if @conf.icon and not(@conf.icon.empty?) then
+		if /\.ico$/ =~ @conf.icon then
+			%Q[<link rel="shortcut icon" href="#{CGI::escapeHTML @conf.icon}">]
+		else
+			%Q[<link rel="icon" href="#{CGI::escapeHTML @conf.icon}">]
+		end
+	else
+		''
+	end
+end
+
+def description_tag
+	if @conf.description and not(@conf.description.empty?) then
+		%Q[<meta name="description" content="#{CGI::escapeHTML @conf.description}">]
+	else
+		''
+	end
+end
+
 def theme_url; 'theme'; end
 
 def css_tag
-	if @conf.theme and @conf.theme.length > 0 then
+	if @mode =~ /conf$/ then
+		css = "#{theme_url}/conf.css"
+	elsif @conf.theme and @conf.theme.length > 0
 		css = "#{theme_url}/#{@conf.theme}/#{@conf.theme}.css"
-		title = css
 	else
 		css = @css
 	end
@@ -262,11 +367,48 @@ def robot_control
 	end
 end
 
+#
+# title of day
+#
+add_title_proc do |date, title|
+	title_of_day( date, title )
+end
+
+def title_of_day( date, title )
+	r = <<-HTML
+	<span class="date">
+	<a href="#{@index}#{anchor( date.strftime( '%Y%m%d' ) )}">#{date.strftime @date_format}</a>
+	</span> 
+	<span class="title">#{title}</span>
+	HTML
+	return r.gsub( /^\t+/, '' ).chomp
+end
+
+add_title_proc do |date, title|
+	nyear_link( date, title )
+end
+
+def nyear_link( date, title )
+	if @conf.show_nyear and @mode != 'nyear' and !@conf.mobile_agent? then
+		y = date.strftime( '%Y' )
+		m = date.strftime( '%m' )
+		d = date.strftime( '%d' )
+		years = @years.find_all {|year, months| months.include? m}
+		if years.length >= 2 then
+			%Q|#{title} <span class="nyear">[<a href="#{@index}#{anchor m + d}" title="#{nyear_diary_title date, years}">#{nyear_diary_label date, years}</a>]</span>|
+		else
+			title
+		end
+	else
+		title
+	end
+end
+
 #
 # make anchor string
 #
 def anchor( s )
-	if /^(\d+)#?([pct]\d*)?$/ =~ s then
+	if /^([\-\d]+)#?([pct]\d*)?$/ =~ s then
 		if $2 then
 			"?date=#$1##$2"
 		else
@@ -277,41 +419,61 @@ def anchor( s )
 	end
 end
 
+#
+# subtitle
+#
+add_subtitle_proc do |date, index, subtitle|
+	subtitle_link( date, index, subtitle )
+end
+
+def subtitle_link( date, index, subtitle )
+	r = ''
+
+	if @conf.mobile_agent? then
+		r << %Q[<A NAME="p#{'%02d' % index}">*</A> ]
+		r << %Q|(#{@author})| if @multi_user and @author and subtitle
+		r << subtitle if subtitle
+	else
+		if date then
+			r << "<a "
+			r << %Q[name="p#{'%02d' % index}" ] if @anchor_name
+			param = "#{date.strftime( '%Y%m%d' )}#p#{'%02d' % index}"
+			r << %Q[href="#{@index}#{anchor param}">#{@conf.section_anchor}</a> ]
+		end
+	
+		r << %Q[(#{@author}) ] if @multi_user and @author and subtitle
+		if subtitle
+			if respond_to?( :category_anchor ) then
+				r << subtitle.sub( /^(\[([^\[]+?)\])+/ ) do
+					$&.gsub( /\[(.*?)\]/ ) do
+						$1.split( /,/ ).collect do |c|
+							category_anchor( "#{c}" )
+						end.join
+					end
+				end
+			else
+				r << subtitle
+			end
+		end
+	end
+	r
+end
+
 #
 # make anchor tag in my diary
 #
 def my( a, str, title = nil )
-	date, noise, frag = a.scan( /^(\d{4}|\d{6}|\d{8})([^\d]*)?#?([pct]\d\d)?$/ )[0]
+	date, noise, frag = a.scan( /^(\d{4}|\d{6}|\d{8}|\d{8}-\d+)([^\d]*)?#?([pct]\d+)?$/ )[0]
 	anc = frag ? "#{date}#{frag}" : date
+	index = /^https?:/ =~ @index ? '' : @conf.base_url
+	index += @index.sub(%r|^\./|, '')
 	if title then
-		%Q[<a href="#{@index}#{anchor anc}" title="#{title}">#{str}</a>]
+		%Q[<a href="#{index}#{anchor anc}" title="#{title}">#{str}</a>]
 	else
-		%Q[<a href="#{@index}#{anchor anc}">#{str}</a>]
+		%Q[<a href="#{index}#{anchor anc}">#{str}</a>]
 	end
 end
 
-#
-# referer of today
-#
-def referer_of_today_short( diary, limit )
-	return '' if not diary or diary.count_referers == 0
-	result = %Q[#{referer_today} | ]
-	diary.each_referer( limit ) do |count,ref|
-		result << %Q[<a href="#{CGI::escapeHTML( ref )}" title="#{CGI::escapeHTML( disp_referer( @referer_table, ref ) )}">#{count}</a> | ]
-	end
-	result
-end
-
-def referer_of_today_long( diary, limit )
-	return '' if not diary or diary.count_referers == 0
-	result = %Q[<div class="caption">#{referer_today}</div>\n]
-	result << %Q[<ul>\n]
-	diary.each_referer( limit ) do |count,ref|
-		result << %Q[<li>#{count} <a href="#{CGI::escapeHTML( ref )}">#{CGI::escapeHTML( disp_referer( @referer_table, ref ) )}</a></li>\n]
-	end
-	result + '</ul>'
-end
-
 #
 # other resources
 #
@@ -332,17 +494,61 @@ def preview_command
 end
 
 #
-# nyear
+# make comment form
 #
-def nyear(ymd)
-	y, m, d = ymd.scan(/^(\d{4})(\d\d)(\d\d)$/)[0]
-	date = Time.local(y, m, d)
-	years = @years.find_all {|year, months| months.include? m}
-	if @mode != 'nyear' and years.length >= 2
-		%Q|[<a href="#{@index}#{anchor m + d}" title="#{nyear_diary_title date, years}">#{nyear_diary_label date, years}</a>]|
-	else
-		""
+def comment_form
+	return '' unless @mode == 'day'
+
+	r = ''
+	unless @conf.hide_comment_form then
+		r = <<-FORM
+			<div class="form">
+		FORM
+		if @diaries[@date.strftime('%Y%m%d')].count_comments( true ) >= @conf.comment_limit_per_day then
+			r << <<-FORM
+				<div class="caption"><a name="c">#{comment_limit_label}</a></div>
+			FORM
+		else
+			r << <<-FORM
+					<div class="caption"><a name="c">#{comment_description}</a></div>
+					<form class="comment" method="post" action="#{@conf.index}"><div>
+					<input type="hidden" name="date" value="#{ @date.strftime( '%Y%m%d' )}">
+					<div class="field name">
+						#{comment_name_label}:<input class="field" name="name" value="#{CGI::escapeHTML( @cgi.cookies['tdiary'][0] || '' )}">
+					</div>
+					<div class="field mail">
+						#{comment_mail_label}:<input class="field" name="mail" value="#{CGI::escapeHTML( @cgi.cookies['tdiary'][1] || '' )}">
+					</div>
+					<div class="textarea">
+						#{comment_body_label}:<textarea name="body" cols="60" rows="5"></textarea>
+					</div>
+					<div class="button">
+						<input type="submit" name="comment" value="#{comment_submit_label}">
+					</div>
+					</div></form>
+			FORM
+		end
+		r << <<-FORM
+			</div>
+		FORM
 	end
+	r
+end
+
+def comment_form_mobile
+	return '' if @conf.hide_comment_form
+	return <<-FORM
+		<HR>
+		<FORM METHOD="POST" ACTION="#{@conf.index}">
+			<INPUT TYPE="HIDDEN" NAME="date" VALUE="#{@date.strftime( '%Y%m%d' )}">
+			<P>#{comment_description_short}<BR>
+			#{comment_name_label_short}: <INPUT NAME="name"><BR>
+			#{comment_mail_label_short}: <INPUT NAME="mail"><BR>
+			#{comment_body_label_short}:<BR>
+			<TEXTAREA NAME="body"></TEXTAREA><BR>
+			<INPUT TYPE="SUBMIT" NAME="comment" value="#{comment_submit_label_short}"></P>
+		</FORM>
+	FORM
 end
 
 #
@@ -351,6 +557,7 @@ end
 def comment_mail_send
 	return unless @comment
 	return unless @conf['comment_mail.enable']
+	return unless @conf['comment_mail.sendhidden'] or @comment.visible?
 
 	case @conf['comment_mail.receivers']
 	when Array
@@ -392,7 +599,7 @@ def comment_mail_send
 	rescue
 		rmail = File::open( "#{::TDiary::PATH}/skel/mail.rtxt" ){|f| f.read }
 	end
-	text = ERbLight::new( rmail.untaint ).result( binding )
+	text = ERB::new( rmail.untaint ).result( binding )
 	receivers.each { |i| i.untaint }
 	comment_mail( text, receivers )
 end
@@ -407,6 +614,7 @@ def comment_mail_basic_setting
 		@conf['comment_mail.enable'] = @cgi.params['comment_mail.enable'][0] == 'true' ? true : false
 		@conf['comment_mail.receivers'] = @cgi.params['comment_mail.receivers'][0].strip.gsub( /[\n\r]+/, ',' )
 		@conf['comment_mail.header'] = @cgi.params['comment_mail.header'][0]
+		@conf['comment_mail.sendhidden'] = @cgi.params['comment_mail.sendhidden'][0] == 'true' ? true : false
 	end
 end
 
@@ -417,17 +625,19 @@ end
 # basic (default)
 def saveconf_default
 	if @mode == 'saveconf' then
+		@conf.html_title = @conf.to_native( @cgi.params['html_title'][0] )
 		@conf.author_name = @conf.to_native( @cgi.params['author_name'][0] )
 		@conf.author_mail = @cgi.params['author_mail'][0]
 		@conf.index_page = @cgi.params['index_page'][0]
-		@conf.hour_offset = @cgi.params['hour_offset'][0].to_f
+		@conf.description = @conf.to_native( @cgi.params['description'][0] )
+		@conf.icon = @cgi.params['icon'][0]
+		@conf.banner = @cgi.params['banner'][0]
 	end
 end
 
 # header/footer (header)
 def saveconf_header
 	if @mode == 'saveconf' then
-		@conf.html_title = @conf.to_native( @cgi.params['html_title'][0] )
 		@conf.header = @conf.to_native( @cgi.params['header'][0] ).gsub( /\r\n/, "\n" ).gsub( /\r/, '' ).sub( /\n+\z/, '' )
 		@conf.footer = @conf.to_native( @cgi.params['footer'][0] ).gsub( /\r\n/, "\n" ).gsub( /\r/, '' ).sub( /\n+\z/, '' )
 	end
@@ -445,12 +655,46 @@ def saveconf_display
 	end
 end
 
-# themes
-def saveconf_theme
-	# dummy
+# timezone
+def saveconf_timezone
+	if @mode == 'saveconf' then
+		@conf.hour_offset = @cgi.params['hour_offset'][0].to_f
+	end
 end
 
-if @mode =~ /^(conf|saveconf)$/ and @cgi.params['conf'][0] == 'theme' then
+# themes
+def conf_theme_list
+	r = ''
+	t = 0
+	@conf_theme_list.each_with_index do |theme, index|
+		if theme[0] == @conf.theme then
+			select = " selected"
+			t = index
+		end
+		r << %Q|<option value="#{theme[0]}"#{select}>#{theme[1]}</option>|
+	end
+	img = t == 0 ? 'nowprinting' : @conf.theme
+	r << <<-HTML
+	</select>
+	<input name="css" size="50" value="#{ @conf.css }">
+	</p>
+	<p><img id="theme_thumbnail" src="http://www.tdiary.org/theme.image/#{img}.jpg" alt="#{@theme_thumbnail_label}"></p>
+	<script language="JavaScript"><!--
+		function changeTheme( image, list ) {
+			var theme = '';
+			if ( list.selectedIndex == 0 ) {
+				theme = 'nowprinting';
+			} else {
+				theme = list.options[list.selectedIndex].value;
+			}
+			image.src = 'http://www.tdiary.org/theme.image/' + theme + '.jpg'
+		}
+	--></script>
+	#{@theme_location_comment unless @conf.mobile_agent?}
+	HTML
+end
+
+def saveconf_theme
 	if @mode == 'saveconf' then
 		@conf.theme = @cgi.params['theme'][0]
 		@conf.css = @cgi.params['css'][0]
@@ -471,28 +715,9 @@ def saveconf_comment
 		@conf.show_comment = @cgi.params['show_comment'][0] == 'true' ? true : false
 		@conf.comment_limit = @cgi.params['comment_limit'][0].to_i
 		@conf.comment_limit = 3 if @conf.comment_limit < 1
-	end
-end
 
-# referer
-def saveconf_referer
-	if @mode == 'saveconf' then
-		@conf.show_referer = @cgi.params['show_referer'][0] == 'true' ? true : false
-		@conf.referer_limit = @cgi.params['referer_limit'][0].to_i
-		@conf.referer_limit = 10 if @conf.referer_limit < 1
-		@conf.referer_day_only = @cgi.params['referer_day_only'][0] == 'true' ? true : false
-		no_referer2 = []
-		@conf.to_native( @cgi.params['no_referer'][0] ).each do |ref|
-			ref.strip!
-			no_referer2 << ref if ref.length > 0
-		end
-		@conf.no_referer2 = no_referer2
-		referer_table2 = []
-		@conf.to_native( @cgi.params['referer_table'][0] ).each do |pair|
-			u, n = pair.sub( /[\r\n]+/, '' ).split( /[ \t]+/, 2 )
-			referer_table2 << [u,n] if u and n
-		end
-		@conf.referer_table2 = referer_table2
+		@conf.comment_limit_per_day = @cgi.params['comment_limit_per_day'][0].to_i
+		@conf.comment_limit_per_day = 0 if @conf.comment_limit_per_day < 0
 	end
 end
 
